<?php

namespace Drupal\terms_export_import\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\terms_export_import\Service\TermsExportImportService;
use Drupal\Core\File\FileSystemInterface;

class TermsImportForm extends FormBase {

    /**
     * @var \Drupal\terms_export_import\Service\TermsExportImportService
     */
    protected $exportImportService;

    public static function create(ContainerInterface $container) {
        $instance = parent::create($container);
        $instance->exportImportService = $container->get('terms_export_import.service');
        return $instance;
    }

    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'terms_import_form';
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state) {
        $form['vocabulary'] = [
            '#type' => 'entity_autocomplete',
            '#title' => $this->t('Vocabulary'),
            '#target_type' => 'taxonomy_vocabulary',
            '#required' => TRUE,
            '#description' => $this->t('Select the vocabulary to import into.'),
        ];

        $form['import_file'] = [
            '#type' => 'file',
            '#title' => $this->t('CSV file'),
            '#description' => $this->t('Upload a CSV file with columns: name,description,parent,weight'),
            '#required' => TRUE,
        ];

        $form['actions'] = [
            '#type' => 'actions',
        ];

        $form['actions']['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Import CSV'),
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state) {
        $file = file_save_upload('import_file', [
            'file_validate_extensions' => ['csv'],
        ]);
        if (!$file) {
            $form_state->setErrorByName('import_file', $this->t('Please upload a valid CSV file.'));
        } else {
            // Move file to temporary location so we can process it.
            $destination = 'public://taxonomy_import_export/' . $file->getFilename();
            $file->setPermanent();
            $file->save();
            $file->move('public://taxonomy_import_export', $file->getFilename());
            $form_state->setValue('uploaded_file', $file);
        }
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state) {
        /** @var \Drupal\file\FileInterface $file */
        $file = $form_state->getValue('uploaded_file');
        if (!$file) {
            $this->messenger()->addError($this->t('File upload error.'));
            return;
        }
        $vocabulary_id = $form_state->getValue('vocabulary');
        $vocab = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->load($vocabulary_id);
        if (!$vocab) {
            $this->messenger()->addError($this->t('Invalid vocabulary selected.'));
            return;
        }

        // Read CSV file content.
        $file_uri = $file->getFileUri();
        $csv_string = file_get_contents($file_uri);

        // Parse CSV into array.
        $terms_data = $this->importExportService->parseCsv($csv_string);

        // Import into taxonomy.
        $this->exportImportService->importTerms($vocab->id(), $terms_data);

        $this->messenger()->addStatus($this->t('Taxonomy terms have been successfully imported.'));
    }
}