<?php

namespace Drupal\terms_export_import\Service;

use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Core\File\FileSystemInterface;

class TermsExportImportService {

    /**
     * Export all terms for a given vocabulary as an array of simple data.
     *
     * @param string $vocabulary_id
     *   The machine name of the vocabulary to export.
     *
     * @return array
     *   An array of term data. Example:
     *   [
     *     [
     *       'name' => 'Term 1',
     *       'description' => 'Description for Term 1',
     *       'parent' => 'Parent Term Name',
     *       'weight' => 0,
     *     ],
     *     ...
     *   ]
     */
    public function exportTerms($vocabulary_id) {
        $terms_data = [];
        $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')
            ->loadTree($vocabulary_id);

        foreach ($terms as $term) {
            $term_obj = Term::load($term->tid);
            $parent_term = '';
            if (!empty($term->parents)) {
                // For simplicity, assume single parent (Drupal supports multiple).
                $parent_id = reset($term->parents);
                if ($parent_id != 0) {
                    $parent_obj = Term::load($parent_id);
                    $parent_term = $parent_obj ? $parent_obj->getName() : '';
                }
            }
            $terms_data[] = [
                'name' => $term_obj->getName(),
                'description' => $term_obj->getDescription(),
                'parent' => $parent_term,
                'weight' => $term_obj->getWeight(),
            ];
        }

        return $terms_data;
    }

    /**
     * Import terms from an array of data into a given vocabulary.
     *
     * @param string $vocabulary_id
     *   The machine name of the vocabulary to import into.
     * @param array $terms_data
     *   Array of terms data (same structure as export).
     */
    public function importTerms($vocabulary_id, array $terms_data) {
        // Validate the vocabulary exists.
        $vocabulary = Vocabulary::load($vocabulary_id);
        if (!$vocabulary) {
            \Drupal::messenger()->addError('Invalid vocabulary ID.');
            return;
        }

        // Key terms by name for parent lookups.
        // Load existing terms to avoid duplicates or handle updates.
        $existing_terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')
            ->loadTree($vocabulary_id);
        $existing_map = [];
        foreach ($existing_terms as $term) {
            $existing_map[$term->name] = $term->tid;
        }

        foreach ($terms_data as $data) {
            $name = isset($data['name']) ? $data['name'] : '';
            $description = isset($data['description']) ? $data['description'] : '';
            $parent_name = isset($data['parent']) ? $data['parent'] : '';
            $weight = isset($data['weight']) ? $data['weight'] : 0;

            // Check if this term already exists.
            if (isset($existing_map[$name])) {
                // Update existing term if desired.
                $term_obj = Term::load($existing_map[$name]);
            }
            else {
                // Create a new term entity.
                $term_obj = Term::create([
                    'name' => $name,
                    'vid' => $vocabulary_id,
                ]);
            }

            // Set (or update) description and weight.
            $term_obj->setDescription($description);
            $term_obj->setWeight($weight);

            // Resolve parent if not empty.
            if ($parent_name && isset($existing_map[$parent_name])) {
                $term_obj->set('parent', [$existing_map[$parent_name]]);
            }
            else {
                $term_obj->set('parent', []);
            }

            $term_obj->save();

            // If new term, add it to $existing_map so child terms can reference it.
            if (!isset($existing_map[$name])) {
                $existing_map[$name] = $term_obj->id();
            }
        }
    }

    /**
     * Convert array of term data to CSV format and return as a string.
     */
    public function convertToCsv(array $terms_data) {
        // First row: headers
        $csv_rows = [];
        $headers = ['name', 'description', 'parent', 'weight'];
        $csv_rows[] = implode(',', $headers);

        foreach ($terms_data as $term) {
            $row = [
                $this->escapeCsv(isset($term['name']) ? $term['name'] : ''),
                $this->escapeCsv(isset($term['description']) ? $term['description'] : ''),
                $this->escapeCsv(isset($term['parent']) ? $term['parent'] : ''),
                isset($term['weight']) ? $term['weight'] : 0,
            ];
            $csv_rows[] = implode(',', $row);
        }

        return implode("\n", $csv_rows);
    }

    /**
     * Parse CSV string into array of term data.
     */
    public function parseCsv($csv_string) {
        $lines = explode("\n", trim($csv_string));
        $data = [];
        // Assume first line are headers
        $headers = [];
        foreach ($lines as $index => $line) {
            $cells = str_getcsv($line);
            if ($index === 0) {
                $headers = $cells;
                continue;
            }
            $row = [];
            foreach ($headers as $i => $header) {
                $row[$header] = $cells[$i] ?? '';
            }
            $data[] = $row;
        }
        return $data;
    }

    /**
     * Simple CSV escaping for strings.
     */
    protected function escapeCsv($string) {
        // If string contains comma, quote, or new line, enclose in quotes.
        if (preg_match('/[,"\r\n]/', $string)) {
            $string = '"' . str_replace('"', '""', $string) . '"';
        }
        return $string;
    }
}